﻿using System.Collections.ObjectModel;
using ThinkGeo.MapSuite.Core;

namespace AdornmentInteractiveOverlaySample
{
    //This class accepts two images. One image for the needle (north arrow) and another optional image for
    //the frame of the compass. Typically, we want to have an image for the frame representing the angles (in degrees or other units)
    //representing to what angle the map is rotated. We could have a regular frame as in a traditional compass.
    //The image for the frame stays fixed as the map rotates whereas the image for the needle rotates with the map to always point
    //to the north.
    class CompassAdornmentLayer : AdornmentLayer
    {
        GeoImage needleImage = null;
        GeoImage frameImage = null;
        float rotateAngle;

        //Use that constructor with only one image for the needle (will work as a simple north arrow)
        public CompassAdornmentLayer(GeoImage needleImage)
        {
            this.needleImage = needleImage;
        }

        //Use that constructor with both images for needle and frame of the compass.
        public CompassAdornmentLayer(GeoImage needleImage, GeoImage frameImage)
        {
            this.needleImage = needleImage;
            this.frameImage = frameImage;
        }

        public GeoImage NeedleImage
        {
            get { return needleImage; }
            set { needleImage = value; }
        }

        public GeoImage FrameImage
        {
            get { return frameImage; }
            set { frameImage = value; }
        }

        //We needs to update that property as the map rotates to have the needle showing the north accurately.
        public float RotateAngle
        {
            get { return rotateAngle; }
            set { rotateAngle = value; }
        }

        protected override void DrawCore(GeoCanvas canvas, Collection<SimpleCandidate> labelsInAllLayers)
        {
            GeoImage adornmentImage = new GeoImage((int)Width, (int)Height);
            RectangleShape scaledWorldExtent = MapEngine.GetDrawingExtent(canvas.CurrentWorldExtent, (int)Width, (int)Height);

            GdiPlusGeoCanvas adornmentCanvas = new GdiPlusGeoCanvas();
            adornmentCanvas.BeginDrawing(adornmentImage, scaledWorldExtent, canvas.MapUnit);

            float imageWidth;
            float imageHeight;
            CalculateImageSize(out imageWidth, out imageHeight, needleImage);

            adornmentCanvas.DrawScreenImage(needleImage, Width / 2, Height / 2, imageWidth, imageHeight, DrawingLevel.LevelOne, 0, 0, rotateAngle);

            //If the frame image has been set, draws image at the size according to the percentage size with no rotation.
            if (frameImage != null)
            {
                CalculateImageSize(out imageWidth, out imageHeight, frameImage);
                adornmentCanvas.DrawScreenImage(frameImage, Width / 2, Height / 2, imageWidth, imageHeight, DrawingLevel.LevelFour, 0, 0, 0);
            }

            adornmentCanvas.EndDrawing();

            ScreenPointF drawingLocation = GetDrawingLocation(canvas, (int)Width, (int)Height);
            canvas.DrawScreenImageWithoutScaling(adornmentImage, drawingLocation.X + Width / 2, drawingLocation.Y + Height / 2, DrawingLevel.LevelOne, 0, 0, 0);

            base.DrawCore(canvas, labelsInAllLayers);
        }

        private void CalculateImageSize(out float imageWidth, out float imageHeight, GeoImage image)
        {
            imageWidth = Width;
            imageHeight = Height;

            int sourceImageWidth = image.GetWidth();
            int sourceImageHeight = image.GetHeight();

            double widthScaleRatio = Width / sourceImageWidth;
            double heightScaleRatio = Height / sourceImageHeight;

            if (widthScaleRatio > heightScaleRatio)
            {
                imageWidth = (float)(sourceImageWidth * heightScaleRatio);
            }
            else
            {
                imageHeight = (float)(sourceImageHeight * widthScaleRatio);
            }
        }
    }
}
